import 'dart:async';

//import 'package:mysql1/mysql1.dart';
import 'package:smart_parking_solutions_common/smart_parking_solutions_common.dart';
import 'package:galileo_mysql/galileo_mysql.dart';
// ignore: implementation_imports
import 'package:smart_parking_solutions_common/src/credentials.dart';

class DataBase {
  static Future<MySqlConnection> getConnection() async {
    final conn = await MySqlConnection.connect(ConnectionSettings(

        ///Localhost will be host server's ip or remain localhost if we use one server
        ///for the db and REST API
        host: 'localhost',
        port: 3306,
        user: Credentials.dbUser,
        db: 'smart_parking',
        password: Credentials.dbSecret));
    return conn;
  }

  static Future<void> initDB() async {
    final conn = await getConnection();
    await conn.query(
        'CREATE TABLE IF NOT EXISTS tbl_user (userID int NOT NULL AUTO_INCREMENT PRIMARY KEY, googleUserID varchar(255), givenName varchar(255), familyName varchar(255), email varchar(255), password varchar(255), handicapped boolean)');
    await conn.query(
        'CREATE TABLE IF NOT EXISTS tbl_booking (bookingID int NOT NULL AUTO_INCREMENT PRIMARY KEY, bayID varchar(255), createdDate DATETIME, startDate DATETIME, endDate DATETIME, ownerFK int, FOREIGN KEY (ownerFK) REFERENCES tbl_user(userID))');
    await conn.query(
        'CREATE TABLE IF NOT EXISTS tbl_tokens (tokenID int NOT NULL AUTO_INCREMENT PRIMARY KEY, tokenValue VARCHAR(255), ownerFK int, FOREIGN KEY (ownerFK) REFERENCES tbl_user(userID))');
    await conn.close();
  }

  ///TODO prevent inserting duplicates
  static Future<Results> createUser(
      {required String? googleUserID,
      required String? givenName,
      required String? familyName,
      required String? email,
      required String? password,
      required bool? handicapped}) async {
    final conn = await getConnection();
    final Results result;
    final searchResult = await search(
        table: 'tbl_user', searchTermVal: {'googleUserID': googleUserID!});
    if (searchResult.isEmpty) {
      result = await conn.query(
          'insert into tbl_user (googleUserID, givenName, familyName, email, password, handicapped) values(?, ?, ?, ?, ?, ?)',
          [googleUserID, givenName, familyName, email, password, handicapped]);
    } else {
      result = await conn.query(
          'update tbl_user set password = ? where googleUserID = ?',
          [password, googleUserID]);
    }

    await conn.close();
    return result;
  }

  static Future<Results> updateUser(
      {required String column,
      required String newVal,
      required User user}) async {
    final conn = await getConnection();
    final result = await conn.query(
        'update tbl_user set $column = $newVal where googleUserID = ${user.googleUserID}');
    await conn.close();
    return result;
  }

  ///tables:<br>
  /// tbl_user, <br>
  /// tbl_booking <br>
  ///searchTermVal example:<br>
  /// {'googleUserID': '${newUser.googleUserID}'}
  static Future<Results> search(
      {required String? table,
      required Map<String, String>? searchTermVal}) async {
    final conn = await getConnection();
    final result = await conn.query(
        'select * from $table where ${searchTermVal!.entries.first.key} = ?',
        [searchTermVal.entries.first.value]);
    await conn.close();
    return result;
  }

  static Future<Results> select({required String? table}) async {
    final conn = await getConnection();
    final result = await conn.query('select * from $table');
    await conn.close();
    return result;
  }

  static Future<Results> createBooking(
      {required String? bayID,
      required DateTime? startDate,
      required DateTime? endDate,
      required User? owner}) async {
    final conn = await getConnection();
    final result = await conn.query(
        'insert into tbl_booking (bayID, createdDate, startDate, endDate, ownerFK) values(?, ?, ?, ?, ?)',
        [
          bayID,
          DateTime.now().toUtc(),
          startDate,
          endDate,
          await owner!.getUserID()
        ]);
    await conn.close();
    return result;
  }

  static Future<Results> createToken(
      {required String? tokenValue,
      required User? owner,
      required DateTime? createdDate}) async {
    final conn = await getConnection();
    final result = await conn.query(
        'insert into tbl_tokens (tokenValue, ownerFK, createdDate) values(?, ?, ?)',
        [tokenValue, await owner!.getUserID(), createdDate!.toUtc()]);
    await conn.close();
    return result;
  }
}
