///Common object used by front and back end service
///which represents a booking for a parking space
class Booking {
  late int ownerFK;
  late String bayID;

  ///The unique ID of the parking bay
  late DateTime createdDate;
  late DateTime startDate;
  late DateTime endDate;

  Booking(
      {required this.ownerFK,
      required this.bayID,
      required this.createdDate,
      required this.startDate,
      required this.endDate});

  Booking.fromDBObj({required var dbBinary}) {
    bayID = dbBinary.fields['bayID'];
    createdDate = dbBinary.fields['createdDate'];
    startDate = dbBinary.fields['startDate'];
    endDate = dbBinary.fields['endDate'];
    ownerFK = dbBinary.fields['ownerFK'];
  }

  Map toJson() {
    final map = {
      'bayID': bayID,
      'createdDate': createdDate.toString(),
      'startDate': startDate.toString(),
      'endDate': endDate.toString(),
      'ownerFK': ownerFK,
    };
    return map;
  }

  bool isOverlapping({required DateTime start, required DateTime end}) {
    //start is after other startDate and end is before other end date (within)
    if ((start.compareTo(startDate) >= 0 && end.compareTo(endDate) <= 0) ||
        //start is before other startDate and end is before other endDate (overlap from the left)
        (start.compareTo(startDate) <= 0 && end.compareTo(endDate) <= 0) ||
        //start is after other startDate but before endDate (overlap from the right)
        (start.compareTo(startDate) >= 0 && start.compareTo(endDate) <= 0) ||
        //start is before startDate and end is before endDate (full overlap)
        (start.compareTo(startDate) <= 0 && end.compareTo(endDate) >= 0)) {
      return true;
    } else {
      return false;
    }
  }
}
